@@ -26,7 +26,9 @@ import logging
 
 from ncclient import manager
 
+from quantum.plugins.cisco.common import cisco_exceptions
 from quantum.plugins.cisco.db import network_db_v2 as cdb
+from quantum.plugins.cisco.db import nexus_db_v2
 from quantum.plugins.cisco.nexus import cisco_nexus_snippets as snipp
 
 
@@ -38,16 +40,54 @@ class CiscoNEXUSDriver():
     Nexus Driver Main Class
     """
     def __init__(self):
-        pass
+        self.connections = {}
+
+    def _edit_config(self, mgr, target='running', config='',
+                     allowed_exc_strs=None):
+        """Modify switch config for a target config type.
+
+        :param mgr: NetConf client manager
+        :param target: Target config type
+        :param config: Configuration string in XML format
+        :param allowed_exc_strs: Exceptions which have any of these strings
+                                 as a subset of their exception message
+                                 (str(exception)) can be ignored
+
+        :raises: NexusConfigFailed
+
+        """
+        if not allowed_exc_strs:
+            allowed_exc_strs = []
+        try:
+            mgr.edit_config(target, config=config)
+        except Exception as e:
+            for exc_str in allowed_exc_strs:
+                if exc_str in str(e):
+                    break
+            else:
+                # Raise a Quantum exception. Include a description of
+                # the original ncclient exception.
+                raise cisco_exceptions.NexusConfigFailed(config=config, exc=e)
 
     def nxos_connect(self, nexus_host, nexus_ssh_port, nexus_user,
                      nexus_password):
-        """
-        Makes the SSH connection to the Nexus Switch
-        """
-        man = manager.connect(host=nexus_host, port=nexus_ssh_port,
-                              username=nexus_user, password=nexus_password)
-        return man
+        """Make SSH connection to the Nexus Switch."""
+        if getattr(self.connections.get(nexus_host), 'connected', None):
+            return self.connections[nexus_host]
+
+        try:
+            man = manager.connect(host=nexus_host,
+                                  port=nexus_ssh_port,
+                                  username=nexus_user,
+                                  password=nexus_password)
+            self.connections[nexus_host] = man
+        except Exception as e:
+            # Raise a Quantum exception. Include a description of
+            # the original ncclient exception.
+            raise cisco_exceptions.NexusConnectFailed(nexus_host=nexus_host,
+                                                      exc=e)
+
+        return self.connections[nexus_host]
 
     def create_xml_snippet(self, cutomized_config):
         """
@@ -57,12 +97,33 @@ class CiscoNEXUSDriver():
         return conf_xml_snippet
 
     def enable_vlan(self, mgr, vlanid, vlanname):
-        """
-        Creates a VLAN on Nexus Switch given the VLAN ID and Name
-        """
-        confstr = snipp.CMD_VLAN_CONF_SNIPPET % (vlanid, vlanname)
-        confstr = self.create_xml_snippet(confstr)
-        mgr.edit_config(target='running', config=confstr)
+        """Create a VLAN on Nexus Switch given the VLAN ID and Name."""
+        confstr = self.create_xml_snippet(
+            snipp.CMD_VLAN_CONF_SNIPPET % (vlanid, vlanname))
+        self._edit_config(mgr, target='running', config=confstr)
+
+        # Enable VLAN active and no-shutdown states. Some versions of
+        # Nexus switch do not allow state changes for the extended VLAN
+        # range (1006-4094), but these errors can be ignored (default
+        # values are appropriate).
+        state_config = [snipp.CMD_VLAN_ACTIVE_SNIPPET,
+                        snipp.CMD_VLAN_NO_SHUTDOWN_SNIPPET]
+        for snippet in state_config:
+            try:
+                confstr = self.create_xml_snippet(snippet % vlanid)
+                self._edit_config(
+                    mgr,
+                    target='running',
+                    config=confstr,
+                    allowed_exc_strs=["Can't modify state for extended",
+                                      "Command is only allowed on VLAN"])
+            except cisco_exceptions.NexusConfigFailed as e:
+                # Rollback VLAN creation
+                try:
+                    self.disable_vlan(mgr, vlanid)
+                finally:
+                    # Re-raise original exception
+                    raise e
 
     def disable_vlan(self, mgr, vlanid):
         """
@@ -70,7 +131,7 @@ class CiscoNEXUSDriver():
         """
         confstr = snipp.CMD_NO_VLAN_CONF_SNIPPET % vlanid
         confstr = self.create_xml_snippet(confstr)
-        mgr.edit_config(target='running', config=confstr)
+        self._edit_config(mgr, target='running', config=confstr)
 
     def enable_port_trunk(self, mgr, interface):
         """
@@ -79,7 +140,7 @@ class CiscoNEXUSDriver():
         confstr = snipp.CMD_PORT_TRUNK % (interface)
         confstr = self.create_xml_snippet(confstr)
         LOG.debug(_("NexusDriver: %s"), confstr)
-        mgr.edit_config(target='running', config=confstr)
+        self._edit_config(mgr, target='running', config=confstr)
 
     def disable_switch_port(self, mgr, interface):
         """
@@ -88,17 +149,22 @@ class CiscoNEXUSDriver():
         confstr = snipp.CMD_NO_SWITCHPORT % (interface)
         confstr = self.create_xml_snippet(confstr)
         LOG.debug(_("NexusDriver: %s"), confstr)
-        mgr.edit_config(target='running', config=confstr)
+        self._edit_config(mgr, target='running', config=confstr)
 
-    def enable_vlan_on_trunk_int(self, mgr, interface, vlanid):
+    def enable_vlan_on_trunk_int(self, mgr, nexus_switch, interface, vlanid):
+        """Enable vlan in trunk interface.
+
+        Enables trunk mode vlan access on an interface on Nexus Switch given
+        VLANID. If one or more VLANs are already configured on this
+        interface, include the 'add' keyword in the interface configuration.
         """
-        Enables trunk mode vlan access an interface on Nexus Switch given
-        VLANID
-        """
-        confstr = snipp.CMD_VLAN_INT_SNIPPET % (interface, vlanid)
-        confstr = self.create_xml_snippet(confstr)
+        if nexus_db_v2.get_port_switch_bindings(interface, nexus_switch):
+            snippet = snipp.CMD_INT_VLAN_ADD_SNIPPET
+        else:
+            snippet = snipp.CMD_INT_VLAN_SNIPPET
+        confstr = self.create_xml_snippet(snippet % (interface, vlanid))
         LOG.debug(_("NexusDriver: %s"), confstr)
-        mgr.edit_config(target='running', config=confstr)
+        self._edit_config(mgr, target='running', config=confstr)
 
     def disable_vlan_on_trunk_int(self, mgr, interface, vlanid):
         """
@@ -108,7 +174,7 @@ class CiscoNEXUSDriver():
         confstr = snipp.CMD_NO_VLAN_INT_SNIPPET % (interface, vlanid)
         confstr = self.create_xml_snippet(confstr)
         LOG.debug(_("NexusDriver: %s"), confstr)
-        mgr.edit_config(target='running', config=confstr)
+        self._edit_config(mgr, target='running', config=confstr)
 
     def create_vlan(self, vlan_name, vlan_id, nexus_host, nexus_user,
                     nexus_password, nexus_ports,
@@ -124,7 +190,7 @@ class CiscoNEXUSDriver():
             vlan_ids = self.build_vlans_cmd()
         LOG.debug(_("NexusDriver VLAN IDs: %s"), vlan_ids)
         for ports in nexus_ports:
-            self.enable_vlan_on_trunk_int(man, ports, vlan_ids)
+            self.enable_vlan_on_trunk_int(man, nexus_host, ports, vlan_ids)
 
     def delete_vlan(self, vlan_id, nexus_host, nexus_user, nexus_password,
                     nexus_ports, nexus_ssh_port):
@@ -160,7 +226,7 @@ class CiscoNEXUSDriver():
         if not vlan_ids:
             vlan_ids = self.build_vlans_cmd()
         for ports in nexus_ports:
-            self.enable_vlan_on_trunk_int(man, ports, vlan_ids)
+            self.enable_vlan_on_trunk_int(man, nexus_host, ports, vlan_ids)
 
     def remove_vlan_int(self, vlan_id, nexus_host, nexus_user, nexus_password,
                         nexus_ports, nexus_ssh_port):
