@@ -26,6 +26,7 @@ from quantum.common import topics
 from quantum import context
 from quantum.db import agents_db
 from quantum.db import agentschedulers_db
+from quantum.db import db_base_plugin_v2
 from quantum.db import dhcp_rpc_base
 from quantum.db import extraroute_db
 from quantum.db import l3_rpc_base
@@ -329,6 +330,15 @@ class NECPluginV2(nec_plugin_base.NECPluginV2Base,
         net = super(NECPluginV2, self).get_network(context, id)
         tenant_id = net['tenant_id']
 
+        # Make sure auto-delete ports on OFC are deleted.
+        filter = {'network_id': [id],
+                  'device_owner': db_base_plugin_v2.AUTO_DELETE_PORT_OWNERS}
+        auto_delete_ports = self.get_ports(context, filters=filter)
+        for port in auto_delete_ports:
+            LOG.debug(_('delete_network(): deleting auto-delete port'
+                        ' from OFC: %s'), port)
+            self.deactivate_port(context, port)
+
         # get packet_filters associated with the network
         if self.packet_filter_enabled:
             filters = dict(network_id=[id])
@@ -675,13 +685,16 @@ class NECPluginV2RPCCallbacks(object):
         session = rpc_context.session
         for p in kwargs.get('port_added', []):
             id = p['id']
-            port = self.plugin.get_port(rpc_context, id)
-            if port and ndb.get_portinfo(session, id):
+            portinfo = ndb.get_portinfo(session, id)
+            if portinfo:
                 ndb.del_portinfo(session, id)
-                self.plugin.deactivate_port(rpc_context, port)
             ndb.add_portinfo(session, id, datapath_id, p['port_no'],
                              mac=p.get('mac', ''))
-            self.plugin.activate_port_if_ready(rpc_context, port)
+            port = self._get_port(rpc_context, id)
+            if port:
+                if portinfo:
+                    self.plugin.deactivate_port(rpc_context, port)
+                self.plugin.activate_port_if_ready(rpc_context, port)
         for id in kwargs.get('port_removed', []):
             portinfo = ndb.get_portinfo(session, id)
             if not portinfo:
@@ -689,7 +702,7 @@ class NECPluginV2RPCCallbacks(object):
                             "due to portinfo for port_id=%s was not "
                             "registered"), id)
                 continue
-            if portinfo.datapath_id is not datapath_id:
+            if portinfo.datapath_id != datapath_id:
                 LOG.debug(_("update_ports(): ignore port_removed message "
                             "received from different host "
                             "(registered_datapath_id=%(registered)s, "
@@ -697,7 +710,13 @@ class NECPluginV2RPCCallbacks(object):
                           {'registered': portinfo.datapath_id,
                            'received': datapath_id})
                 continue
-            port = self.plugin.get_port(rpc_context, id)
+            ndb.del_portinfo(session, id)
+            port = self._get_port(rpc_context, id)
             if port:
-                ndb.del_portinfo(session, id)
                 self.plugin.deactivate_port(rpc_context, port)
+
+    def _get_port(self, context, port_id):
+        try:
+            return self.plugin.get_port(context, port_id)
+        except q_exc.PortNotFound:
+            return None
