@@ -20,22 +20,33 @@
 # @author: Aaron Rosen, Nicira Networks, Inc.
 # @author: Bob Kukura, Red Hat, Inc.
 
-import logging
-import os
 import sys
 
+from oslo.config import cfg
+
+from quantum.agent import securitygroups_rpc as sg_rpc
+from quantum.api.rpc.agentnotifiers import dhcp_rpc_agent_api
+from quantum.api.rpc.agentnotifiers import l3_rpc_agent_api
 from quantum.api.v2 import attributes
 from quantum.common import constants as q_const
 from quantum.common import exceptions as q_exc
+from quantum.common import rpc as q_rpc
 from quantum.common import topics
+from quantum.db import agents_db
+from quantum.db import agentschedulers_db
 from quantum.db import db_base_plugin_v2
 from quantum.db import dhcp_rpc_base
-from quantum.db import l3_db
+from quantum.db import extraroute_db
+from quantum.db import l3_rpc_base
+# NOTE: quota_db cannot be removed, it is for db model
+from quantum.db import quota_db
+from quantum.db import securitygroups_rpc_base as sg_db_rpc
+from quantum.extensions import portbindings
 from quantum.extensions import providernet as provider
-from quantum.openstack.common import context
-from quantum.openstack.common import cfg
+from quantum.extensions import securitygroup as ext_sg
+from quantum.openstack.common import importutils
+from quantum.openstack.common import log as logging
 from quantum.openstack.common import rpc
-from quantum.openstack.common.rpc import dispatcher
 from quantum.openstack.common.rpc import proxy
 from quantum.plugins.openvswitch.common import config
 from quantum.plugins.openvswitch.common import constants
@@ -46,13 +57,17 @@ from quantum import policy
 LOG = logging.getLogger(__name__)
 
 
-class OVSRpcCallbacks(dhcp_rpc_base.DhcpRpcCallbackMixin):
+class OVSRpcCallbacks(dhcp_rpc_base.DhcpRpcCallbackMixin,
+                      l3_rpc_base.L3RpcCallbackMixin,
+                      sg_db_rpc.SecurityGroupServerRpcCallbackMixin):
 
-    # Set RPC API version to 1.0 by default.
-    RPC_API_VERSION = '1.0'
+    # history
+    #   1.0 Initial version
+    #   1.1 Support Security Group RPC
 
-    def __init__(self, rpc_context, notifier):
-        self.rpc_context = rpc_context
+    RPC_API_VERSION = '1.1'
+
+    def __init__(self, notifier):
         self.notifier = notifier
 
     def create_rpc_dispatcher(self):
@@ -61,13 +76,22 @@ class OVSRpcCallbacks(dhcp_rpc_base.DhcpRpcCallbackMixin):
         If a manager would like to set an rpc API version, or support more than
         one class as the target of rpc messages, override this method.
         '''
-        return dispatcher.RpcDispatcher([self])
+        return q_rpc.PluginRpcDispatcher([self,
+                                          agents_db.AgentExtRpcCallback()])
+
+    @classmethod
+    def get_port_from_device(cls, device):
+        port = ovs_db_v2.get_port_from_device(device)
+        if port:
+            port['device'] = device
+        return port
 
     def get_device_details(self, rpc_context, **kwargs):
         """Agent requests device details"""
         agent_id = kwargs.get('agent_id')
         device = kwargs.get('device')
-        LOG.debug("Device %s details requested from %s", device, agent_id)
+        LOG.debug(_("Device %(device)s details requested from %(agent_id)s"),
+                  locals())
         port = ovs_db_v2.get_port(device)
         if port:
             binding = ovs_db_v2.get_network_binding(None, port['network_id'])
@@ -78,11 +102,13 @@ class OVSRpcCallbacks(dhcp_rpc_base.DhcpRpcCallbackMixin):
                      'network_type': binding.network_type,
                      'segmentation_id': binding.segmentation_id,
                      'physical_network': binding.physical_network}
-            # Set the port status to UP
-            ovs_db_v2.set_port_status(port['id'], q_const.PORT_STATUS_ACTIVE)
+            new_status = (q_const.PORT_STATUS_ACTIVE if port['admin_state_up']
+                          else q_const.PORT_STATUS_DOWN)
+            if port['status'] != new_status:
+                ovs_db_v2.set_port_status(port['id'], new_status)
         else:
             entry = {'device': device}
-            LOG.debug("%s can not be found in database", device)
+            LOG.debug(_("%s can not be found in database"), device)
         return entry
 
     def update_device_down(self, rpc_context, **kwargs):
@@ -90,19 +116,35 @@ class OVSRpcCallbacks(dhcp_rpc_base.DhcpRpcCallbackMixin):
         # (TODO) garyk - live migration and port status
         agent_id = kwargs.get('agent_id')
         device = kwargs.get('device')
-        LOG.debug("Device %s no longer exists on %s", device, agent_id)
+        LOG.debug(_("Device %(device)s no longer exists on %(agent_id)s"),
+                  locals())
         port = ovs_db_v2.get_port(device)
         if port:
             entry = {'device': device,
                      'exists': True}
-            # Set port status to DOWN
-            ovs_db_v2.set_port_status(port['id'], q_const.PORT_STATUS_DOWN)
+            if port['status'] != q_const.PORT_STATUS_DOWN:
+                # Set port status to DOWN
+                ovs_db_v2.set_port_status(port['id'], q_const.PORT_STATUS_DOWN)
         else:
             entry = {'device': device,
                      'exists': False}
-            LOG.debug("%s can not be found in database", device)
+            LOG.debug(_("%s can not be found in database"), device)
         return entry
 
+    def update_device_up(self, rpc_context, **kwargs):
+        """Device is up on agent"""
+        agent_id = kwargs.get('agent_id')
+        device = kwargs.get('device')
+        LOG.debug(_("Device %(device)s up on %(agent_id)s"),
+                  locals())
+        port = ovs_db_v2.get_port(device)
+        if port:
+            if port['status'] != q_const.PORT_STATUS_ACTIVE:
+                ovs_db_v2.set_port_status(port['id'],
+                                          q_const.PORT_STATUS_ACTIVE)
+        else:
+            LOG.debug(_("%s can not be found in database"), device)
+
     def tunnel_sync(self, rpc_context, **kwargs):
         """Update new tunnel.
 
@@ -116,14 +158,15 @@ class OVSRpcCallbacks(dhcp_rpc_base.DhcpRpcCallbackMixin):
         entry = dict()
         entry['tunnels'] = tunnels
         # Notify all other listening agents
-        self.notifier.tunnel_update(self.rpc_context, tunnel.ip_address,
+        self.notifier.tunnel_update(rpc_context, tunnel.ip_address,
                                     tunnel.id)
         # Return the list of tunnels IP's to the agent
         return entry
 
 
-class AgentNotifierApi(proxy.RpcProxy):
-    '''Agent side of the linux bridge rpc API.
+class AgentNotifierApi(proxy.RpcProxy,
+                       sg_rpc.SecurityGroupAgentRpcApiMixin):
+    '''Agent side of the openvswitch rpc API.
 
     API version history:
         1.0 - Initial version.
@@ -170,7 +213,10 @@ class AgentNotifierApi(proxy.RpcProxy):
 
 
 class OVSQuantumPluginV2(db_base_plugin_v2.QuantumDbPluginV2,
-                         l3_db.L3_NAT_db_mixin):
+                         extraroute_db.ExtraRoute_db_mixin,
+                         sg_db_rpc.SecurityGroupServerRpcMixin,
+                         agentschedulers_db.AgentSchedulerDbMixin):
+
     """Implement the Quantum abstractions using Open vSwitch.
 
     Depending on whether tunneling is enabled, either a GRE tunnel or
@@ -183,13 +229,34 @@ class OVSQuantumPluginV2(db_base_plugin_v2.QuantumDbPluginV2,
     handled, by adding support for extended attributes to the
     QuantumDbPluginV2 base class. When that occurs, this class should
     be updated to take advantage of it.
+
+    The port binding extension enables an external application relay
+    information to and from the plugin.
     """
 
     # This attribute specifies whether the plugin supports or not
-    # bulk operations. Name mangling is used in order to ensure it
-    # is qualified by class
+    # bulk/pagination/sorting operations. Name mangling is used in
+    # order to ensure it is qualified by class
     __native_bulk_support = True
-    supported_extension_aliases = ["provider", "router"]
+    __native_pagination_support = True
+    __native_sorting_support = True
+
+    _supported_extension_aliases = ["provider", "router",
+                                    "binding", "quotas", "security-group",
+                                    "agent", "extraroute", "agent_scheduler"]
+
+    @property
+    def supported_extension_aliases(self):
+        if not hasattr(self, '_aliases'):
+            aliases = self._supported_extension_aliases[:]
+            sg_rpc.disable_security_group_extension_if_noop_driver(aliases)
+            self._aliases = aliases
+        return self._aliases
+
+    network_view = "extension:provider_network:view"
+    network_set = "extension:provider_network:set"
+    binding_view = "extension:port_binding:view"
+    binding_set = "extension:port_binding:set"
 
     def __init__(self, configfile=None):
         ovs_db_v2.initialize()
@@ -200,8 +267,8 @@ class OVSQuantumPluginV2(db_base_plugin_v2.QuantumDbPluginV2,
                                             constants.TYPE_VLAN,
                                             constants.TYPE_GRE,
                                             constants.TYPE_NONE]:
-            LOG.error("Invalid tenant_network_type: %s. "
-                      "Agent terminated!",
+            LOG.error(_("Invalid tenant_network_type: %s. "
+                      "Agent terminated!"),
                       self.tenant_network_type)
             sys.exit(1)
         self.enable_tunneling = cfg.CONF.OVS.enable_tunneling
@@ -210,20 +277,23 @@ class OVSQuantumPluginV2(db_base_plugin_v2.QuantumDbPluginV2,
             self._parse_tunnel_id_ranges()
             ovs_db_v2.sync_tunnel_allocations(self.tunnel_id_ranges)
         elif self.tenant_network_type == constants.TYPE_GRE:
-            LOG.error("Tunneling disabled but tenant_network_type is 'gre'. "
-                      "Agent terminated!")
+            LOG.error(_("Tunneling disabled but tenant_network_type is 'gre'. "
+                      "Agent terminated!"))
             sys.exit(1)
-        self.agent_rpc = cfg.CONF.AGENT.rpc
         self.setup_rpc()
+        self.network_scheduler = importutils.import_object(
+            cfg.CONF.network_scheduler_driver)
+        self.router_scheduler = importutils.import_object(
+            cfg.CONF.router_scheduler_driver)
 
     def setup_rpc(self):
         # RPC support
         self.topic = topics.PLUGIN
-        self.rpc_context = context.RequestContext('quantum', 'quantum',
-                                                  is_admin=False)
         self.conn = rpc.create_connection(new=True)
         self.notifier = AgentNotifierApi(topics.AGENT)
-        self.callbacks = OVSRpcCallbacks(self.rpc_context, self.notifier)
+        self.dhcp_agent_notifier = dhcp_rpc_agent_api.DhcpAgentNotifyAPI()
+        self.l3_agent_notifier = l3_rpc_agent_api.L3AgentNotify
+        self.callbacks = OVSRpcCallbacks(self.notifier)
         self.dispatcher = self.callbacks.create_rpc_dispatcher()
         self.conn.create_consumer(self.topic, self.dispatcher,
                                   fanout=False)
@@ -241,13 +311,13 @@ class OVSQuantumPluginV2(db_base_plugin_v2.QuantumDbPluginV2,
                                                  int(vlan_min),
                                                  int(vlan_max))
                 except ValueError as ex:
-                    LOG.error("Invalid network VLAN range: '%s' - %s. "
-                              "Agent terminated!",
-                              entry, ex)
+                    LOG.error(_("Invalid network VLAN range: "
+                                "'%(range)s' - %(e)s. Agent terminated!"),
+                              {'range': entry, 'e': ex})
                     sys.exit(1)
             else:
                 self._add_network(entry)
-        LOG.info("Network VLAN ranges: %s", self.network_vlan_ranges)
+        LOG.info(_("Network VLAN ranges: %s"), self.network_vlan_ranges)
 
     def _add_network_vlan_range(self, physical_network, vlan_min, vlan_max):
         self._add_network(physical_network)
@@ -264,26 +334,23 @@ class OVSQuantumPluginV2(db_base_plugin_v2.QuantumDbPluginV2,
                 tun_min, tun_max = entry.split(':')
                 self.tunnel_id_ranges.append((int(tun_min), int(tun_max)))
             except ValueError as ex:
-                LOG.error("Invalid tunnel ID range: '%s' - %s. "
-                          "Agent terminated!", entry, ex)
+                LOG.error(_("Invalid tunnel ID range: "
+                            "'%(range)s' - %(e)s. Agent terminated!"),
+                          {'range': entry, 'e': ex})
                 sys.exit(1)
-        LOG.info("Tunnel ID ranges: %s", self.tunnel_id_ranges)
+        LOG.info(_("Tunnel ID ranges: %s"), self.tunnel_id_ranges)
 
     # TODO(rkukura) Use core mechanism for attribute authorization
     # when available.
 
-    def _check_provider_view_auth(self, context, network):
-        return policy.check(context,
-                            "extension:provider_network:view",
-                            network)
+    def _check_view_auth(self, context, resource, action):
+        return policy.check(context, action, resource)
 
-    def _enforce_provider_set_auth(self, context, network):
-        return policy.enforce(context,
-                              "extension:provider_network:set",
-                              network)
+    def _enforce_set_auth(self, context, resource, action):
+        policy.enforce(context, action, resource)
 
     def _extend_network_dict_provider(self, context, network):
-        if self._check_provider_view_auth(context, network):
+        if self._check_view_auth(context, network, self.network_view):
             binding = ovs_db_v2.get_network_binding(context.session,
                                                     network['id'])
             network[provider.NETWORK_TYPE] = binding.network_type
@@ -314,7 +381,7 @@ class OVSQuantumPluginV2(db_base_plugin_v2.QuantumDbPluginV2,
             return (None, None, None)
 
         # Authorize before exposing plugin details to client
-        self._enforce_provider_set_auth(context, attrs)
+        self._enforce_set_auth(context, attrs, self.network_set)
 
         if not network_type_set:
             msg = _("provider:network_type required")
@@ -360,14 +427,14 @@ class OVSQuantumPluginV2(db_base_plugin_v2.QuantumDbPluginV2,
             else:
                 segmentation_id = None
         else:
-            msg = _("provider:network_type %s not supported" % network_type)
+            msg = _("provider:network_type %s not supported") % network_type
             raise q_exc.InvalidInput(error_message=msg)
 
         if network_type in [constants.TYPE_VLAN, constants.TYPE_FLAT]:
             if physical_network_set:
                 if physical_network not in self.network_vlan_ranges:
-                    msg = _("unknown provider:physical_network %s" %
-                            physical_network)
+                    msg = _("Unknown provider:physical_network "
+                            "%s") % physical_network
                     raise q_exc.InvalidInput(error_message=msg)
             elif 'default' in self.network_vlan_ranges:
                 physical_network = 'default'
@@ -391,9 +458,9 @@ class OVSQuantumPluginV2(db_base_plugin_v2.QuantumDbPluginV2,
             return
 
         # Authorize before exposing plugin details to client
-        self._enforce_provider_set_auth(context, attrs)
+        self._enforce_set_auth(context, attrs, self.network_set)
 
-        msg = _("plugin does not support updating provider attributes")
+        msg = _("Plugin does not support updating provider attributes")
         raise q_exc.InvalidInput(error_message=msg)
 
     def create_network(self, context, network):
@@ -402,6 +469,11 @@ class OVSQuantumPluginV2(db_base_plugin_v2.QuantumDbPluginV2,
                                                           network['network'])
 
         session = context.session
+        #set up default security groups
+        tenant_id = self._get_tenant_id_for_create(
+            context, network['network'])
+        self._ensure_default_security_group(context, tenant_id)
+
         with session.begin(subtransactions=True):
             if not network_type:
                 # tenant network
@@ -431,7 +503,7 @@ class OVSQuantumPluginV2(db_base_plugin_v2.QuantumDbPluginV2,
             self._extend_network_dict_provider(context, net)
             self._extend_network_dict_l3(context, net)
             # note - exception will rollback entire transaction
-        LOG.debug("Created network: %s", net['id'])
+        LOG.debug(_("Created network: %s"), net['id'])
         return net
 
     def update_network(self, context, id, network):
@@ -461,8 +533,7 @@ class OVSQuantumPluginV2(db_base_plugin_v2.QuantumDbPluginV2,
                                        self.network_vlan_ranges)
             # the network_binding record is deleted via cascade from
             # the network record, so explicit removal is not necessary
-        if self.agent_rpc:
-            self.notifier.network_delete(self.rpc_context, id)
+        self.notifier.network_delete(context, id)
 
     def get_network(self, context, id, fields=None):
         session = context.session
@@ -473,41 +544,103 @@ class OVSQuantumPluginV2(db_base_plugin_v2.QuantumDbPluginV2,
             self._extend_network_dict_l3(context, net)
         return self._fields(net, fields)
 
-    def get_networks(self, context, filters=None, fields=None):
+    def get_networks(self, context, filters=None, fields=None,
+                     sorts=None,
+                     limit=None, marker=None, page_reverse=False):
         session = context.session
         with session.begin(subtransactions=True):
-            nets = super(OVSQuantumPluginV2, self).get_networks(context,
-                                                                filters,
-                                                                None)
+            nets = super(OVSQuantumPluginV2,
+                         self).get_networks(context, filters, None, sorts,
+                                            limit, marker, page_reverse)
             for net in nets:
                 self._extend_network_dict_provider(context, net)
                 self._extend_network_dict_l3(context, net)
 
-            # TODO(rkukura): Filter on extended provider attributes.
-            nets = self._filter_nets_l3(context, nets, filters)
-
         return [self._fields(net, fields) for net in nets]
 
-    def update_port(self, context, id, port):
-        if self.agent_rpc:
-            original_port = super(OVSQuantumPluginV2, self).get_port(context,
-                                                                     id)
-        port = super(OVSQuantumPluginV2, self).update_port(context, id, port)
-        if self.agent_rpc:
-            if original_port['admin_state_up'] != port['admin_state_up']:
-                binding = ovs_db_v2.get_network_binding(None,
-                                                        port['network_id'])
-                self.notifier.port_update(self.rpc_context, port,
-                                          binding.network_type,
-                                          binding.segmentation_id,
-                                          binding.physical_network)
+    def _extend_port_dict_binding(self, context, port):
+        if self._check_view_auth(context, port, self.binding_view):
+            port[portbindings.VIF_TYPE] = portbindings.VIF_TYPE_OVS
+            port[portbindings.CAPABILITIES] = {
+                portbindings.CAP_PORT_FILTER:
+                'security-group' in self.supported_extension_aliases}
         return port
 
+    def create_port(self, context, port):
+        # Set port status as 'DOWN'. This will be updated by agent
+        port['port']['status'] = q_const.PORT_STATUS_DOWN
+        session = context.session
+        with session.begin(subtransactions=True):
+            self._ensure_default_security_group_on_port(context, port)
+            sgids = self._get_security_groups_on_port(context, port)
+            port = super(OVSQuantumPluginV2, self).create_port(context, port)
+            self._process_port_create_security_group(
+                context, port['id'], sgids)
+            self._extend_port_dict_security_group(context, port)
+        self.notify_security_groups_member_updated(context, port)
+        return self._extend_port_dict_binding(context, port)
+
+    def get_port(self, context, id, fields=None):
+        with context.session.begin(subtransactions=True):
+            port = super(OVSQuantumPluginV2, self).get_port(context,
+                                                            id, fields)
+            self._extend_port_dict_security_group(context, port)
+            self._extend_port_dict_binding(context, port)
+        return self._fields(port, fields)
+
+    def get_ports(self, context, filters=None, fields=None,
+                  sorts=None, limit=None, marker=None,
+                  page_reverse=False):
+        with context.session.begin(subtransactions=True):
+            ports = super(OVSQuantumPluginV2, self).get_ports(
+                context, filters, fields, sorts, limit, marker,
+                page_reverse)
+            #TODO(nati) filter by security group
+            for port in ports:
+                self._extend_port_dict_security_group(context, port)
+                self._extend_port_dict_binding(context, port)
+        return [self._fields(port, fields) for port in ports]
+
+    def update_port(self, context, id, port):
+        session = context.session
+
+        need_port_update_notify = False
+        with session.begin(subtransactions=True):
+            original_port = super(OVSQuantumPluginV2, self).get_port(
+                context, id)
+            updated_port = super(OVSQuantumPluginV2, self).update_port(
+                context, id, port)
+            need_port_update_notify = self.update_security_group_on_port(
+                context, id, port, original_port, updated_port)
+
+        need_port_update_notify |= self.is_security_group_member_updated(
+            context, original_port, updated_port)
+
+        if original_port['admin_state_up'] != updated_port['admin_state_up']:
+            need_port_update_notify = True
+
+        if need_port_update_notify:
+            binding = ovs_db_v2.get_network_binding(None,
+                                                    updated_port['network_id'])
+            self.notifier.port_update(context, updated_port,
+                                      binding.network_type,
+                                      binding.segmentation_id,
+                                      binding.physical_network)
+
+        return self._extend_port_dict_binding(context, updated_port)
+
     def delete_port(self, context, id, l3_port_check=True):
 
         # if needed, check to see if this is a port owned by
         # and l3-router.  If so, we should prevent deletion.
         if l3_port_check:
             self.prevent_l3_port_deletion(context, id)
-        self.disassociate_floatingips(context, id)
-        return super(OVSQuantumPluginV2, self).delete_port(context, id)
+
+        session = context.session
+        with session.begin(subtransactions=True):
+            self.disassociate_floatingips(context, id)
+            port = self.get_port(context, id)
+            self._delete_port_security_group_bindings(context, id)
+            super(OVSQuantumPluginV2, self).delete_port(context, id)
+
+        self.notify_security_groups_member_updated(context, port)
